@@ -1,7 +1,7 @@
 /*
     This file is part of RoboJournal.
     Copyright (c) 2012 by Will Kraft <pwizard@gmail.com>.
-    MADE IN USA
+    
 
     RoboJournal is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,6 +15,12 @@
 
     You should have received a copy of the GNU General Public License
     along with RoboJournal.  If not, see <http://www.gnu.org/licenses/>.
+
+    UPDATE 8/18/13: Use "@mysql" as the global connection name for all regular
+    database transactions. Functions that require root access should use something
+    different (e.g. @create, @MASTER , etc.) to avoid breaking the current journal
+    connection.
+
   */
 
 
@@ -31,13 +37,17 @@
 #include <QTime>
 #include <QRegExp>
 #include "sql/sqlshield.h"
+#include <QApplication>
+#include "ui/mainwindow.h"
+#include <QtNetwork/QNetworkInterface>
+#include <QSqlError>
 
 
 QSqlDatabase MySQLCore::db;
 int MySQLCore::dialogX;
 int MySQLCore::dialogY;
 int MySQLCore::ID;
-QString MySQLCore::error_code;
+QString MySQLCore::error_code; // not really used
 
 QString MySQLCore::recordnum;
 
@@ -47,6 +57,60 @@ MySQLCore::MySQLCore()
 {
 }
 
+//################################################################################################
+// New feature for 0.5, 6/23/13 -- Will Kraft.
+// Use the root account to grant permission to use "database" to user "username". Create username account if necessary.
+bool MySQLCore::GrantPermissions(bool create_account, QString database, QString user_host, QString username, QString port,
+                                 QString root_host, QString user_password, QString root_password){
+
+    bool success=true; //success should be true unless something goes wrong and changes it to false
+    int db_port=port.toInt();
+
+    QSqlDatabase db2 = QSqlDatabase::addDatabase("QMYSQL","@grant");
+    db2.setHostName(root_host);
+    db2.setPort(db_port);
+    db2.setUserName("root");
+    db2.setPassword(root_password);
+    db2.open();
+
+    // Bugfix 8/21/12: Save old login data in case login fails. That way other connections won't get broken after
+    // exiting JournalSelector
+    old_username2=db2.userName();
+    old_password2=db2.password();
+
+    if(create_account){
+        QSqlQuery q("CREATE USER \'" + username + "\'@\'" + user_host +
+                    "\' IDENTIFIED BY \'" + user_password + "\'",db2);
+        q.exec();
+    }
+
+    QSqlQuery q2("GRANT INSERT,DELETE,UPDATE,SELECT,RELOAD ON " + database + ".entries TO \'" +
+                 username + "\'@\'" + user_host + "\'",db2);
+    success=q2.exec();
+
+    // Flush privileges on host after grant so the new changes are available immediately (Will Kraft 4/6/14).
+    QSqlQuery q3("FLUSH PRIVILEGES",db2);
+    q3.exec();
+
+    db2.close();
+
+    // if login fails, restore the old values before closing the database. That way, the connections in the rest of
+    // the app won't get broken.
+    if(!success){
+        db2.setUserName(old_username2);
+        db2.setPassword(old_password2);
+        // clean up
+        old_password2.clear();
+        old_username2.clear();
+    }
+
+    // Bugfix (8/19/13) Change db2's association so we can remove it without creating warnings.
+    db2 = QSqlDatabase();
+    db2.removeDatabase("@grant");
+
+    return success;
+}
+
 //################################################################################################
 // New tag reminder function for 0.4.1 (added 2/25/13). This code does a lookup for rows where tags are null
 // (i.e. this entry has not been tagged yet) and returns the results as a qlist full of qstringlists.
@@ -55,16 +119,17 @@ QList<QStringList> MySQLCore::NullSearch(){
 
     QList<QStringList> values;
 
-    db.open();
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
 
-    QSqlQuery q("SELECT id, title, day, month, year FROM entries WHERE tags=\"NULL\" ORDER BY id DESC");
+    QSqlQuery q("SELECT id, title, day, month, year FROM entries WHERE tags=\"NULL\" ORDER BY id DESC",db2);
 
     q.exec();
 
 
 
     while(q.next()){
-            QStringList entry;
+        QStringList entry;
         QVariant v0=q.value(0); //id;
         QVariant v1=q.value(1); //title
         QVariant v2=q.value(2); //day
@@ -95,13 +160,13 @@ QList<QStringList> MySQLCore::NullSearch(){
             break;
         }
 
-       entry << id << title << datestamp;
+        entry << id << title << datestamp;
 
         //append the row to the main list
         values.append(entry);
     }
 
-    db.close();
+    db2.close();
 
     return values;
 }
@@ -125,9 +190,11 @@ QList<QStringList> MySQLCore::NullSearch(){
 
 QStringList MySQLCore::Get_Entry_New(QString id){
 
-    db.open();
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+    //db.open();
 
-    QSqlQuery q("SELECT title, month, day, year, time, tags, body FROM entries WHERE id=?");
+    QSqlQuery q("SELECT title, month, day, year, time, tags, body FROM entries WHERE id=?",db2);
     q.bindValue(0,id);
     q.exec();
 
@@ -191,15 +258,15 @@ QStringList MySQLCore::Get_Entry_New(QString id){
         entry.append(v6.toString()); // body
     }
 
-    db.close();
+    db2.close();
 
     return entry;
 }
 
 
-QStringList MySQLCore::Get_Latest_New(QString id){
+//QStringList MySQLCore::Get_Latest_New(QString id){
 
-}
+//}
 
 //################################################################################################
 // Search database, new for 0.4
@@ -207,7 +274,9 @@ QList<QStringList> MySQLCore::SearchDatabase(QString searchterm, int index, QStr
 
     QList<QStringList> results;
 
-    db.open();
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+    //db.open();
     QString query;
 
 
@@ -267,7 +336,7 @@ QList<QStringList> MySQLCore::SearchDatabase(QString searchterm, int index, QStr
     // input.
     SQLShield s;
     QString sanitized=s.Break_Injections(query);
-    QSqlQuery q(sanitized);
+    QSqlQuery q(sanitized,db2);
     q.exec();
 
 
@@ -391,7 +460,7 @@ QList<QStringList> MySQLCore::SearchDatabase(QString searchterm, int index, QStr
         results.append(entry);
     }
 
-    db.close();
+    db2.close();
 
     return results;
 }
@@ -403,7 +472,9 @@ QList<QStringList> MySQLCore::DumpDatabase(bool asc){
 
     QList<QStringList> journal;
 
-    db.open();
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+    //db.open();
     QString query;
 
     // sort ascending
@@ -415,7 +486,7 @@ QList<QStringList> MySQLCore::DumpDatabase(bool asc){
         query="SELECT title,month,day,year,tags,body,time FROM entries ORDER BY id DESC";
     }
 
-    QSqlQuery q(query);
+    QSqlQuery q(query,db2);
     q.exec();
 
     while(q.next()){
@@ -445,49 +516,46 @@ QList<QStringList> MySQLCore::DumpDatabase(bool asc){
         entry.clear();
     }
 
-    db.close();
+    db2.close();
 
     return journal;
 }
 
 //################################################################################################
-
 // Get list of databases for JournalSelector class
-QStringList MySQLCore::GetDatabaseList(QString hostname, QString port, QString username, QString password){
+QStringList MySQLCore::GetDatabaseList(QString hostname, QString port, QString username, QString password, bool silentMode){
     using namespace std;
 
-    // Bugfix 8/8/12: Save old login data in case login fails. That way other connections won't get broken after
-    // exiting JournalSelector
-    old_username=db.userName();
-    old_password=db.password();
+    QSqlDatabase db2 = QSqlDatabase::addDatabase("QMYSQL","@MASTER");
 
     int socket=port.toInt();
-    //cout << "Data: " << hostname.toStdString() + ";" +username.toStdString() + ";" +
-    //password.toStdString(); + ";" + socket;
 
     // use information_schema database if Buffer::database_name is null.
     // that way, there will not be a segfault due to a a null variable.
-    // we don't want ot do anything with that database, just link to it.
+    // we don't want to do anything with that database, just link to it.
     if(Buffer::database_name.isEmpty()){
         Buffer::database_name="information_schema";
     }
 
-    //QSqlDatabase db2;
-    db = QSqlDatabase::addDatabase("QMYSQL");
-    db.setHostName(hostname);
-    db.setPort(socket);
-    db.setUserName(username);
-    db.setPassword(password);
-    db.setDatabaseName(Buffer::database_name);
+
+    db2.setHostName(hostname);
+    db2.setPort(socket);
+    db2.setUserName(username);
+    db2.setPassword(password);
+    db2.setDatabaseName(Buffer::database_name);
 
     QStringList journals;
 
+    // Bugfix 8/8/12: Save old login data in case login fails. That way other connections won't get broken after
+    // exiting JournalSelector
+    old_username=db2.userName();
+    old_password=db2.password();
 
-    bool success=db.open();
+    bool success=db2.open();
 
 
     if(success){
-        QSqlQuery q("SHOW DATABASES");
+        QSqlQuery q("SHOW DATABASES",db2);
 
         q.exec();
 
@@ -507,21 +575,23 @@ QStringList MySQLCore::GetDatabaseList(QString hostname, QString port, QString u
 
         if(journals.size()==0){
             QMessageBox b;
-            b.information(NULL,"RoboJournal","RoboJournal found no databases on <b>" + hostname + "</b>.");
+            b.information(MainWindow::mw,"RoboJournal","RoboJournal found no databases belonging to user \""
+                          + username + "@" + hostname + "\". You should use the <b>Assign Permissions</b> tool "
+                          "to give this account access to one or more databases on the host.");
         }
     }
     else{
 
         // If there's no MySQL, display an error
-        if(!db.isDriverAvailable("QMYSQL")){
+        if(!db2.isDriverAvailable("QMYSQL")){
             success=false;
             cout << "OUTPUT: MySQL support was not found! MySQL functionality disabled." << endl;
             QMessageBox a;
-            a.critical(NULL,"RoboJournal","RoboJournal could not find or use the Qt MySQL driver. "
+            a.critical(MainWindow::mw,"RoboJournal","RoboJournal could not find or use the Qt MySQL driver. "
                        "This problem occurs if the Qt environment used to compile RoboJournal was built without MySQL support.");
         }
 
-        if(db.isOpenError()){
+        if((db2.isOpenError()) && (!silentMode)){
             QMessageBox m;
             QString reason;
 
@@ -533,7 +603,7 @@ QStringList MySQLCore::GetDatabaseList(QString hostname, QString port, QString u
                         "</b> from this computer? If so, make sure you entered the correct username/password and try again.";
             }
 
-            m.critical(NULL,"RoboJournal","RoboJournal could not connect to  <b>" +
+            m.critical(MainWindow::mw,"RoboJournal","RoboJournal could not connect to  <b>" +
                        hostname + "</b>.<br><br>" + reason );
         }
     }
@@ -541,33 +611,42 @@ QStringList MySQLCore::GetDatabaseList(QString hostname, QString port, QString u
     // if login fails, restore the old values before closing the database. That way, the connections in the rest of
     // the app won't get broken.
     if(!success){
-        db.setUserName(old_username);
-        db.setPassword(old_password);
+        db2.setUserName(old_username);
+        db2.setPassword(old_password);
         // clean up
         old_password.clear();
         old_username.clear();
     }
 
-    db.close();
+    db2.close();
+
+    // Bugfix (8/19/13) Change db2's association so we can remove it without creating warnings.
+    db2 = QSqlDatabase();
+    db2.removeDatabase("@MASTER");
+
     return journals;
 }
+
 //################################################################################################
 // upgrade journal to RoboJournal >0.1 compatible version by adding [time varchar(5)] column
 bool MySQLCore::UpgradeJournal(QString root_pass){
-    db = QSqlDatabase::addDatabase("QMYSQL");
-    db.setHostName(Buffer::host);
-    db.setDatabaseName(Buffer::database_name);
-    db.setPort(Buffer::databaseport);
-    db.setUserName("root");
-    db.setPassword(root_pass);
-    db.open();
 
-    QSqlQuery q("ALTER TABLE entries ADD time VARCHAR(5)");
+    QSqlDatabase db2 = QSqlDatabase::addDatabase("QMYSQL","@MASTER");
+    db2.setHostName(Buffer::host);
+    db2.setDatabaseName(Buffer::database_name);
+    db2.setPort(Buffer::databaseport);
+    db2.setUserName("root");
+    db2.setPassword(root_pass);
+    db2.open();
+
+    QSqlQuery q("ALTER TABLE entries ADD time VARCHAR(5)",db2);
     bool success=q.exec();
 
+    db2.close();
 
-
-    db.close();
+    // Bugfix (8/19/13) Change db2's association so we can remove it without creating warnings.
+    db2 = QSqlDatabase();
+    db2.removeDatabase("@MASTER");
 
     return success;
 }
@@ -578,9 +657,12 @@ bool MySQLCore::UpgradeJournal(QString root_pass){
 
 bool MySQLCore::SanityCheck(){
     using namespace std;
-    db.open();
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
     QString query="DESCRIBE entries";
-    QSqlQuery q(query);
+    QSqlQuery q(query,db2);
     q.exec();
     bool is_sane=false;
 
@@ -622,7 +704,8 @@ bool MySQLCore::SanityCheck(){
         }
 
     }
-    db.close();
+
+    db2.close();
     return is_sane;
 }
 
@@ -631,6 +714,7 @@ bool MySQLCore::SanityCheck(){
 bool MySQLCore::Connect(){
 
     using namespace std;
+
     cout << "OUTPUT: Attempting MySQL database connection on \"" << Buffer::host.toStdString()
          << "\" as user \"" << Buffer::username.toStdString() << "\"...";
 
@@ -640,49 +724,79 @@ bool MySQLCore::Connect(){
     QString password=Buffer::password;
     int port=Buffer::databaseport;
 
-    db = QSqlDatabase::addDatabase("QMYSQL");
-    db.setHostName(host);
-    db.setPort(port);
-    db.setUserName(user);
-    db.setDatabaseName(database);
-    db.setPassword(password);
+    QSqlDatabase db2 = QSqlDatabase::addDatabase("QMYSQL","@mysql");
+    db2.setHostName(host);
+    db2.setPort(port);
+    db2.setUserName(user);
+    db2.setDatabaseName(database);
+    db2.setPassword(password);
 
     if(Buffer::SSL){
-        db.setConnectOptions("CLIENT_SSL=1");
+        db2.setConnectOptions("CLIENT_SSL=1");
     }
 
 
-    bool success=db.open();
+    bool success=db2.open();
 
     // The idea here just to see if we can create a connection at all.
     // if success returns true, the login form class DBLogin proceeds and the rest of the
     // program is unlocked. Other parts of the program can then call MysqlCore directly as needed.
-
-
     if(success){
         cout << "SUCCESS!"<< endl;
-        db.close();
-
-
+        db2.close();
+        return true;
     }
     else{
         cout << "FAILED!" << endl;
-        db.close();
+
+        // Bugfix (8/25/13): Destroy the old connection and allow the user to reconnect.
+        db2.close();
+
+
+
+        if(db2.isOpenError()){
+            QMessageBox m;
+
+            QString reason;
+
+            if((Buffer::host=="localhost") || (Buffer::host=="127.0.0.1")){
+                reason="Make sure you entered the correct username/password and try again.";
+            }
+            else{
+                reason="Are you allowed to access <b>" + Buffer::database_name +
+                        "</b> from this computer? If so, make sure you entered the correct username/password and try again.";
+            }
+
+            m.critical(MainWindow::mw,"RoboJournal","RoboJournal could not connect to  <b>" +
+                       Buffer::database_name + "</b>@<b>" +
+                       Buffer::host + "</b>.<br><br>" + reason );
+        }
+
+        // check to make sure the MYSQL driver is installed, return error if false
+        // If you're using a static build of QT you're probably never going to see this error
+        if(!db2.isDriverAvailable("QMYSQL")){
+            QMessageBox j;
+            j.critical(MainWindow::mw,"RoboJournal","RoboJournal could not find the MySQL driver. This problem likely occurred"
+                       " because the Qt libraries on this computer were compiled incorrectly or are incomplete. RoboJournal"
+                       " cannot use MySQL databases until this issue is resolved.");
+        }
+
+        QSqlDatabase db2;
+        db2.removeDatabase("@mysql");
+
+        return false;
     }
-
-
-
-
-    return success;
 }
+
 //################################################################################################
 // I don't think this is even being used anymore
 void MySQLCore::Disconnect(){
     using namespace std;
     cout << "OUTPUT: Attempting to close MySQL connection...";
 
-    db.removeDatabase("QMYSQL");
-    db.close();
+    QSqlDatabase db2;
+    db2.removeDatabase("@mysql");
+    db2.close();
 
     cout << "Done!" << endl;
 
@@ -690,29 +804,37 @@ void MySQLCore::Disconnect(){
 }
 //################################################################################################
 // Update a row with new data. This is invoked from the Editor class if editmode==true.
-bool MySQLCore::Update(QString title, int month, int day, int year,  QString body, QString id){
+bool MySQLCore::Update(QString title, int month, int day, int year,  QString body, QString id, QString tags){
     using namespace std;
-    db.open();
 
-    QSqlQuery update("UPDATE entries set title=?, month=?, day=?, year=?, body=? WHERE id=" + id);
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
+    QSqlQuery update("UPDATE entries set title=?, month=?, day=?, year=?, tags=?, body=? WHERE id=" + id,db2);
     update.bindValue(0, title);
     update.bindValue(1, month);
     update.bindValue(2, day);
     update.bindValue(3, year);
-    update.bindValue(4, body);
+    update.bindValue(4, tags); // Add tags parameter for 0.5, 6/29/13.
+    update.bindValue(5, body);
+
 
     bool success=update.exec();
-    db.close();
+    db2.close();
 
     return success;
 }
 //################################################################################################
 bool MySQLCore::UpdateTags(QString tag_data, QString id){
-    db.open();
-    QSqlQuery tag("UPDATE entries SET tags=? WHERE id=" + id);
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
+    QSqlQuery tag("UPDATE entries SET tags=? WHERE id=" + id,db2);
     tag.bindValue(0,tag_data);
     bool success=tag.exec();
-    db.close();
+
+    db2.close();
     return success;
 }
 
@@ -720,13 +842,15 @@ bool MySQLCore::UpdateTags(QString tag_data, QString id){
 //################################################################################################
 bool MySQLCore::AddEntry(){
     using namespace std;
-    db.open();
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
 
     QString title=Editor::title;
     int month=Editor::month;
     int day=Editor::day;
     int year=Editor::year;
-    QString tags="Null";
+    QString tags=Editor::tags;
     QString body=Editor::body;
 
 
@@ -734,7 +858,7 @@ bool MySQLCore::AddEntry(){
     // if the user opts to record time, write extra time field
     if(Buffer::keep_time){
         QString insert="INSERT INTO entries(title, month, day, year, tags, body, time) VALUES(?,?,?,?,?,?,?)";
-        QSqlQuery addentry(insert);
+        QSqlQuery addentry(insert,db2);
         addentry.bindValue(0, title);
         addentry.bindValue(1, month);
         addentry.bindValue(2, day);
@@ -748,13 +872,10 @@ bool MySQLCore::AddEntry(){
         addentry.bindValue(6, timestamp);
         //cout << "OUTPUT: Current timestamp: " << timestamp.toStdString() << endl;
         success=addentry.exec();
-
-
-
     }
     else{
         QString insert="INSERT INTO entries(title, month, day, year, tags, body) VALUES(?,?,?,?,?,?)";
-        QSqlQuery addentry(insert);
+        QSqlQuery addentry(insert,db2);
         addentry.bindValue(0, title);
         addentry.bindValue(1, month);
         addentry.bindValue(2, day);
@@ -763,35 +884,42 @@ bool MySQLCore::AddEntry(){
         addentry.bindValue(5, body);
         success=addentry.exec();
     }
-    db.close();
+
+    db2.close();
     return success;
 }
 
 //################################################################################################
 bool MySQLCore::DeleteEntry(QString id){
 
-    db.open();
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
     QString query="DELETE FROM entries WHERE id=" + id;
-    QSqlQuery q(query);
+    QSqlQuery q(query,db2);
 
     bool success=q.exec();
-    db.close();
+    db2.close();
     return success;
 }
 //################################################################################################
 // Get day and year based on entry
 QString MySQLCore::TimeStamp(QString id){
     using namespace std;
-    db.open();
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
     QString query="SELECT month,day,year FROM entries WHERE id=" + id;
-    //cout << query.toStdString() << endl;
-    QSqlQuery q(query);
+
+    QSqlQuery q(query,db2);
 
     q.exec();
     QString month;
     QString day;
     QString year;
     QString timestamp;
+
     while(q.next()){
         QVariant Vmonth=q.value(0);
         month=Vmonth.toString();
@@ -845,7 +973,7 @@ QString MySQLCore::TimeStamp(QString id){
         //cout << entry.toStdString() << endl;
     }
 
-    db.close();
+    db2.close();
 
     return timestamp;
 }
@@ -856,10 +984,13 @@ QString MySQLCore::TimeStamp(QString id){
 // index should be the value from the second hidden column.
 QString MySQLCore::RetrieveEntry(QString id){
     using namespace std;
-    db.open();
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
     QString query="SELECT body FROM entries WHERE id=" + id;
     //cout << query.toStdString() << endl;
-    QSqlQuery q(query);
+    QSqlQuery q(query,db2);
 
     q.exec();
     QString entry;
@@ -869,7 +1000,7 @@ QString MySQLCore::RetrieveEntry(QString id){
         //cout << entry.toStdString() << endl;
     }
 
-    db.close();
+    db2.close();
 
     return entry;
 }
@@ -884,15 +1015,18 @@ QString MySQLCore::RetrieveEntry(QString id){
 
 QList<QString> MySQLCore::RetrieveEntryFull(QString id){
     using namespace std;
-    db.open();
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
     QString query="SELECT title, month, day, year, tags, body FROM entries WHERE id=" + id;
     //cout << query.toStdString() << endl;
 
-    QSqlQuery q(query);
+    QSqlQuery q(query,db2);
 
     q.exec();
 
-    db.close();
+    db2.close();
 
     QList<QString> EntryArray;
 
@@ -921,10 +1055,12 @@ QList<QString> MySQLCore::RetrieveEntryFull(QString id){
 // Get the most recent entry from the db and return it as a QString
 QString MySQLCore::Recent(){
 
-    db.open();
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+    //db.open();
     QString q="SELECT id,body FROM entries ORDER BY id DESC";
-    QSqlQuery get;
-    get.prepare(q);
+    QSqlQuery get(q,db2);
+
     get.exec();
     get.first();
     QVariant result=get.value(1);
@@ -937,23 +1073,26 @@ QString MySQLCore::Recent(){
 
     recordnum=id.toString();
 
-    db.close();
+    db2.close();
 
     return body;
 }
 //################################################################################################
 QString MySQLCore::GetPrevious(){
-    db.open();
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
     ID--;
     QString q="SELECT body FROM entries where id=?";
 
-    QSqlQuery get(q);
+    QSqlQuery get(q,db2);
     get.bindValue(0, ID);
     get.exec();
     QVariant result=get.value(0);
-
     QString body=result.toString();
-    db.close();
+
+    db2.close();
     return body;
 
 }
@@ -963,9 +1102,11 @@ QList<QString> MySQLCore::getYear(){
     using namespace std;
     QList <QString> YearList;
 
-    db.open();
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
     QString query="SELECT year FROM entries ORDER BY id DESC";
-    QSqlQuery getyear(query);
+    QSqlQuery getyear(query,db2);
     getyear.exec();
 
     int pos=0;
@@ -1019,7 +1160,7 @@ QList<QString> MySQLCore::getYear(){
 
     }
 
-    db.close();
+    db2.close();
 
     return YearList;
 }
@@ -1027,9 +1168,12 @@ QList<QString> MySQLCore::getYear(){
 QList<QString> MySQLCore::getMonth(QString nextyear){
     using namespace std;
     QList <QString> MonthList;
-    db.open();
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
     QString query="SELECT month FROM entries WHERE year=" + nextyear +  " ORDER BY id DESC";
-    QSqlQuery getmonth(query);
+    QSqlQuery getmonth(query,db2);
     getmonth.exec();
 
 
@@ -1047,7 +1191,8 @@ QList<QString> MySQLCore::getMonth(QString nextyear){
         recent=month;
 
     }
-    db.close();
+
+    db2.close();
     return MonthList;
 }
 //################################################################################################
@@ -1055,10 +1200,12 @@ QList<QString> MySQLCore::getDay(QString itemmonth, QString nextyear){
     using namespace std;
     QList <QString> DayList;
 
-    db.open();
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
     QString query="SELECT day FROM entries WHERE MONTH=" + itemmonth + " AND year=" + nextyear + " ORDER BY id DESC";
     //cout << query.toStdString() << endl;
-    QSqlQuery getday(query);
+    QSqlQuery getday(query,db2);
     getday.exec();
     int pos=0;
     QString previous;
@@ -1074,14 +1221,18 @@ QList<QString> MySQLCore::getDay(QString itemmonth, QString nextyear){
         previous=day;
 
     }
-    db.close();
+
+    db2.close();
 
     return DayList;
 
 }
 //################################################################################################
 QList<QString> MySQLCore::getEntries(QString itemday, QString itemmonth){
-    db.open();
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
     QList <QString> EntryList;
 
     // Even though we want to display results from newest-> oldest,
@@ -1090,7 +1241,7 @@ QList<QString> MySQLCore::getEntries(QString itemday, QString itemmonth){
 
     QString query="SELECT title,id FROM entries WHERE day=" + itemday + " AND month=" +
             itemmonth + " ORDER BY id ASC";
-    QSqlQuery getentry(query);
+    QSqlQuery getentry(query,db2);
 
 
     getentry.exec();
@@ -1104,7 +1255,6 @@ QList<QString> MySQLCore::getEntries(QString itemday, QString itemmonth){
         QVariant rownum=getentry.value(1);
         QString id=rownum.toString();
 
-
         QString previous=NULL;
 
         entry=entry+"|"+id;
@@ -1117,7 +1267,8 @@ QList<QString> MySQLCore::getEntries(QString itemday, QString itemmonth){
         }
         previous=entry;
     }
-    db.close();
+
+    db2.close();
 
     return EntryList;
 
@@ -1127,10 +1278,13 @@ QList<QString> MySQLCore::getEntries(QString itemday, QString itemmonth){
 QString MySQLCore::GetTimestamp(QString id){
 
     using namespace std;
-    db.open();
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
     QString query="SELECT time FROM entries WHERE id=" + id;
     //cout << query.toStdString() << endl;
-    QSqlQuery q(query);
+    QSqlQuery q(query,db2);
 
     q.exec();
     QString timestamp="test";
@@ -1153,8 +1307,7 @@ QString MySQLCore::GetTimestamp(QString id){
         }
     }
 
-    //cout << "Timestamp: " + timestamp.toStdString() << endl;
-    db.close();
+    db2.close();
     return timestamp;
 }
 
@@ -1162,10 +1315,13 @@ QString MySQLCore::GetTimestamp(QString id){
 // Get title from database
 QString MySQLCore::GetTitle(QString id){
     using namespace std;
-    db.open();
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
     QString query="SELECT title FROM entries WHERE id=" + id;
-    //cout << query.toStdString() << endl;
-    QSqlQuery q(query);
+
+    QSqlQuery q(query,db2);
 
     q.exec();
     QString title;
@@ -1175,7 +1331,7 @@ QString MySQLCore::GetTitle(QString id){
         //cout << entry.toStdString() << endl;
     }
 
-    db.close();
+    db2.close();
 
     return title;
 }
@@ -1183,8 +1339,12 @@ QString MySQLCore::GetTitle(QString id){
 // Get tags from the database
 QString MySQLCore::GetTags(QString id){
     using namespace std;
-    db.open();
-    QSqlQuery q("SELECT tags FROM entries WHERE id=" + id);
+
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
+    QSqlQuery q("SELECT tags FROM entries WHERE id=" + id,db2);
     q.exec();
 
     QString tags;
@@ -1194,7 +1354,7 @@ QString MySQLCore::GetTags(QString id){
         //cout << entry.toStdString() << endl;
     }
 
-    db.close();
+    db2.close();
     return tags;
 }
 //################################################################################################
@@ -1203,8 +1363,10 @@ QList<QString> MySQLCore::TagSearch(){
     using namespace std;
     QList<QString> tags;
 
-    db.open();
-    QSqlQuery q("SELECT tags FROM entries");
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
+    QSqlQuery q("SELECT tags FROM entries",db2);
     q.exec();
 
     QString next_tag;
@@ -1218,21 +1380,25 @@ QList<QString> MySQLCore::TagSearch(){
             tags.append(next_tag);
         }
     }
-    db.close();
+
+    db2.close();
     return tags;
 }
 
 //################################################################################################
 // This creates the list of all entries in the database so we can browse backward and forward
-QList<QString> MySQLCore::Create_ID_List(int year_range){
+QList<QString> MySQLCore::Create_ID_List(){
     using namespace std;
     QList <QString> IDList;
-    db.open();
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
+
     QString query="SELECT id FROM entries ORDER BY id ASC";
-    //cout << "Creating id list";
-    QSqlQuery id(query);
+
+    QSqlQuery id(query,db2);
     id.exec();
-    year_range=0;  // not used, so set to null for now
 
     int pos=0;
     while(id.next()){
@@ -1242,7 +1408,8 @@ QList<QString> MySQLCore::Create_ID_List(int year_range){
         IDList.insert(pos, entry);
         pos++;
     }
-    db.close();
+
+    db2.close();
 
     return IDList;
 }
@@ -1260,10 +1427,14 @@ QList<QString> MySQLCore::Create_ID_List(int year_range){
 QList<QString> MySQLCore::getEntriesMonth(QString month, QString year){
     using namespace std;
     QList <QString> EntryList;
-    db.open();
+
+    QSqlDatabase db2=QSqlDatabase::database("@mysql");
+    db2.open();
+
     QString query="SELECT id,title,day FROM entries WHERE month=" +
             month + " AND year="+ year +" ORDER BY ID DESC";
-    QSqlQuery b(query);
+    QSqlQuery b(query,db2);
+
     b.exec();
 
     int pos=0;
@@ -1281,7 +1452,8 @@ QList<QString> MySQLCore::getEntriesMonth(QString month, QString year){
         EntryList.insert(pos, item);
         pos++;
     }
-    db.close();
+
+    db2.close();
     return EntryList;
 }
 
@@ -1294,28 +1466,32 @@ QList<QString> MySQLCore::getEntriesMonth(QString month, QString year){
   */
 
 bool MySQLCore::CreateDatabase(QString host,QString root_pass, QString db_name,
-                               QString port, QString newuser, QString newuser_pass){
+                               QString port, QString newuser, QString newuser_pass,
+                               bool on_remote_host)
+{
+
+    QSqlDatabase db2 = QSqlDatabase::addDatabase("QMYSQL", "@create");
     using namespace std;
 
     // Bugfix 8/21/12: Save old login data in case login fails. That way other connections won't get broken after
     // exiting JournalSelector
-    old_username=db.userName();
-    old_password=db.password();
+    old_username=db2.userName();
+    old_password=db2.password();
 
     bool success=true; //success should be true unless something goes wrong and changes it to false
     bool exists=false;
 
     int db_port=port.toInt();
 
-    db = QSqlDatabase::addDatabase("QMYSQL");
-    db.setHostName(host);
-    db.setPort(db_port);
-    db.setUserName("root");
-    db.setPassword(root_pass);
-    db.open();
+
+    db2.setHostName(host);
+    db2.setPort(db_port);
+    db2.setUserName("root");
+    db2.setPassword(root_pass);
+    db2.open();
 
 
-    QSqlQuery duplicate_check("SHOW DATABASES");
+    QSqlQuery duplicate_check("SHOW DATABASES",db2);
     duplicate_check.exec();
 
 
@@ -1331,63 +1507,102 @@ bool MySQLCore::CreateDatabase(QString host,QString root_pass, QString db_name,
             break;
         }
     }
-    db.close();
+    db2.close();
 
     // only proceed if the other database does not exist
     if(!exists){
-        db.open();
+        db2.open();
         // create database
-        QSqlQuery create_db("CREATE DATABASE IF NOT EXISTS " + db_name);
+        QSqlQuery create_db("CREATE DATABASE IF NOT EXISTS " + db_name,db2);
         create_db.exec();
 
-        db.close();
+        db2.close();
 
-        db.setDatabaseName(db_name);
-        db.open();
+        db2.setDatabaseName(db_name);
+        db2.open();
         // create the entries table now
 
         QSqlQuery create_entries("CREATE TABLE entries(id int not null auto_increment, "
                                  "primary key (id), title VARCHAR(500), month VARCHAR(2), "
                                  "day VARCHAR(2), year VARCHAR(4), tags VARCHAR(300), "
-                                 "body TEXT, time VARCHAR(5))");
+                                 "body TEXT, time VARCHAR(5))",db2);
 
         create_entries.exec();
-        db.close();
+        db2.close();
 
         // Grant rights to new user
-        db.open();
+        db2.open();
         QString grant="GRANT SELECT, INSERT, DROP, UPDATE, DELETE ON " + db_name +
-                ".* TO " + newuser + "@'" + host + "' IDENTIFIED BY '" + newuser_pass + "'";
-
-        QSqlQuery grant_rights(grant);
-
+                ".* TO '" + newuser + "'@'" + host + "' IDENTIFIED BY '" + newuser_pass + "'";
+        QSqlQuery grant_rights(grant,db2);
         grant_rights.exec();
 
-        db.close();
+
+        // 0.5 Bugfix: Grant persissions to user if database is located on remote host --Will Kraft (6/7/14).
+        if(on_remote_host){
+
+             QList<QHostAddress> if_list = QNetworkInterface::allAddresses();
+             QStringList ipaddr;
+
+             // Get a list of all ip addresses on the system and save the ones that are for LAN (192.168.x.y)
+             for(int nIter=0; nIter<if_list.count(); nIter++)
+             {
+                   QString addr=if_list[nIter].toString();
+
+                   if(addr.contains("192.168."))
+                       ipaddr << addr;
+             }
+
+             // Grant permessions to every IP in the list so the user wil lbe able to access their journal from
+             // the current remote computer.
+
+             for(int i=0; i<ipaddr.count(); i++){
+
+                 host=ipaddr.at(i);
+
+                 QSqlQuery grant_remote_rights("GRANT SELECT, INSERT, DROP, UPDATE, DELETE ON `" + db_name +
+                                               "`.* TO '" + newuser + "'@'" + host + "' IDENTIFIED BY '" +
+                                               newuser_pass + "'; FLUSH PRIVILEGES",db2);
+                 grant_remote_rights.exec();
+
+                 cout << "OUTPUT: Granting rights to remote database '" << newuser.toStdString() << "'@'"
+                      << host.toStdString() << "'." << endl;
+
+                 qDebug() << grant_remote_rights.lastError();
+             }
+        }
+
+        // Flush the privileges so the new Grants start working
+        QSqlQuery flush("FLUSH PRIVILEGES",db2);
+        flush.exec();
+
+
+        db2.close();
     }
 
     // show warning if database exists
     if(exists){
         cout << "OUTPUT: " << db_name.toStdString() << " already exists! The original database was not replaced." << endl;
         QMessageBox s;
-        s.critical(NULL,"RoboJournal","<b>" + db_name + "</b> already exists on <b>" + host +
+        s.critical(MainWindow::mw,"RoboJournal","<b>" + db_name + "</b> already exists on <b>" + host +
                    "</b>! The existing <b>" + db_name + "</b> database was not changed or replaced.");
+        return true;
     }
 
 
     // show an error if we failed to connect
-    if(db.isOpenError()){
-        success=false;
+    if(db2.isOpenError()){
         cout << "OUTPUT: RoboJournal could not connect. Are you using the right username and password?" << endl;
+        return false;
     }
 
     // If there's no MySQL, display an error
-    if(!db.isDriverAvailable("QMYSQL")){
-        success=false;
+    if(!db2.isDriverAvailable("QMYSQL")){
         cout << "OUTPUT: MySQL support was not found! MySQL functionality disabled." << endl;
         QMessageBox a;
-        a.critical(NULL,"RoboJournal","RoboJournal could not find or use the Qt MySQL driver. "
+        a.critical(MainWindow::mw,"RoboJournal","RoboJournal could not find or use the Qt MySQL driver. "
                    "This problem occurs if the Qt environment used to compile RoboJournal was built without MySQL support.");
+        return false;
     }
 
     // if everything went ok provide feedback.
@@ -1402,13 +1617,16 @@ bool MySQLCore::CreateDatabase(QString host,QString root_pass, QString db_name,
     // if login fails, restore the old values before closing the database. That way, the connections in the rest of
     // the app won't get broken.
     if(!success){
-        db.setUserName(old_username);
-        db.setPassword(old_password);
+        db2.setUserName(old_username);
+        db2.setPassword(old_password);
         // clean up
         old_password.clear();
         old_username.clear();
     }
 
+    // Bugfix (8/19/13) Change db2's association so we can remove it without creating warnings.
+    db2 = QSqlDatabase();
+    db2.removeDatabase("@create");
 
     return success;
 }
