@@ -1,7 +1,7 @@
 /*
     This file is part of RoboJournal.
     Copyright (c) 2012 by Will Kraft <pwizard@gmail.com>.
-    MADE IN USA
+    
 
     RoboJournal is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,11 +28,11 @@ class SQLiteCore
 {
 public:
     SQLiteCore();
-    bool Connect();
+    bool Connect(QString dbname);
     bool Disconnect();
     bool AddEntry();
     bool RemoveEntry();
-    bool CreateDB();
+    bool CreateDB(QString dbname);
 
     static QSqlDatabase db;
 
