@@ -1,7 +1,7 @@
 /*
     This file is part of RoboJournal.
     Copyright (c) 2012 by Will Kraft <pwizard@gmail.com>.
-    MADE IN USA
+    
 
     RoboJournal is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #define FIRSTRUN_H
 
 #include <QDialog>
+#include <QAbstractButton>
 
 namespace Ui {
     class FirstRun;
@@ -37,14 +38,12 @@ public:
 
 private slots:
 
-
-
-
-
     void on_buttonBox_accepted();
 
     void on_buttonBox_rejected();
 
+    void on_buttonBox_clicked(QAbstractButton *button);
+
 private:
     Ui::FirstRun *ui;
 
